// Copyright (c) 2025 shibedrill
// SPDX-License-Identifier: GPL-3.0-or-later

#![allow(clippy::missing_safety_doc)]
#![allow(dead_code)]

use core::arch::asm;

pub fn read_cr0() -> u64 {
    let cr0: u64;
    unsafe { asm!("mov {0:r}, cr0", out(reg) cr0) }
    cr0
}

pub fn read_cr2() -> usize {
    let cr2: usize;
    unsafe { asm!("mov {0:r}, cr2", out(reg) cr2) }
    cr2
}

pub fn read_cr3() -> u64 {
    let cr3: u64;
    unsafe { asm!("mov {0:r}, cr3", out(reg) cr3) }
    cr3
}

pub fn write_cr3(val: u64) {
    unsafe { asm!("mov cr3, {0:r}", in(reg) val) }
}

pub fn interrupt_disable() {
    unsafe {
        asm!("cli");
    }
}

pub fn interrupt_enable() {
    unsafe {
        asm!("sti");
    }
}

pub fn halt() {
    unsafe {
        asm!("hlt");
    }
}

pub fn nop() {
    unsafe {
        asm!("nop");
    }
}

pub fn port_read_u8(port: u16) -> u8 {
    let result: u8;
    unsafe {
        asm! {
            "in al dx",
            in("dx") port,
            out("al") result
        }
    }
    result
}

pub fn port_read_u16(port: u16) -> u16 {
    let result: u16;
    unsafe {
        asm! {
            "in ax dx",
            in("dx") port,
            out("ax") result
        }
    }
    result
}

pub fn port_read_u32(port: u16) -> u32 {
    let result: u32;
    unsafe {
        asm! {
            "in eax dx",
            in("dx") port,
            out("eax") result
        }
    }
    result
}

pub fn port_write_u8(port: u16, data: u8) {
    unsafe {
        asm! {
            "out dx, al",
            in("dx") port,
            in("al") data,
        }
    }
}

pub fn port_write_u16(port: u16, data: u16) {
    unsafe {
        asm! {
            "out dx, ax",
            in("dx") port,
            in("ax") data,
        }
    }
}

pub fn port_write_u32(port: u16, data: u32) {
    unsafe {
        asm! {
            "out dx eax",
            in("dx") port,
            in("eax") data,
        }
    }
}
