use num_derive::FromPrimitive;
use num_traits::FromPrimitive;

use crate::arch;

#[derive(FromPrimitive, Debug)]
pub enum Syscall {
    Exit,
    Pid,
    Message,
    MemMap,
    MemUnmap,
    Resolve,
    Spawn,
    Version,
}

#[derive(FromPrimitive, Debug)]
pub enum SyscallStatus {
    Success,
    NoSuchCall,
    NoSuchProcess,
    NoSuchServer,
    NoPermission,
    OutOfMemory,
    Aborted,
    Unspecified,
    Unknown,
}

pub struct UnknownStatus {}

impl From<u64> for SyscallStatus {
    fn from(value: u64) -> Self {
        SyscallStatus::from_u64(value).unwrap_or(Self::Unknown)
    }
}

pub fn exit(code: u64) -> SyscallStatus {
    arch::syscall_impl::caller_syscall_1(Syscall::Exit as u64, code).into()
}
